@@ -15,7 +15,7 @@
  * - psutil.Process.io_counters read count is always 0
  * - psutil.Process.io_counters may not be available on older AIX versions
  * - psutil.Process.threads may not be available on older AIX versions
- # - psutil.net_io_counters may not be available on older AIX versions
+ * - psutil.net_io_counters may not be available on older AIX versions
  * - reading basic process info may fail or return incorrect values when
  *   process is starting (see IBM APAR IV58499 - fixed in newer AIX versions)
  * - sockets and pipes may not be counted in num_fds (fixed in newer AIX
@@ -29,7 +29,6 @@
  */
 
 #include <Python.h>
-
 #include <sys/limits.h>
 #include <sys/proc.h>
 #include <sys/procfs.h>
@@ -51,11 +50,11 @@
 #include <libperfstat.h>
 #include <unistd.h>
 
+#include "_psutil_common.h"
+#include "_psutil_posix.h"
 #include "arch/aix/ifaddrs.h"
 #include "arch/aix/net_connections.h"
 #include "arch/aix/common.h"
-#include "_psutil_common.h"
-#include "_psutil_posix.h"
 
 
 #define TV2DOUBLE(t)   (((t).tv_nsec * 0.000000001) + (t).tv_sec)
@@ -462,7 +461,7 @@ psutil_proc_num_ctx_switches(PyObject *self, PyObject *args) {
 
     /* finished iteration without finding requested pid */
     free(processes);
-    return NoSuchProcess("");
+    return NoSuchProcess("psutil_read_process_table (no PID found)");
 }
 
 
@@ -498,11 +497,11 @@ psutil_users(PyObject *self, PyObject *args) {
         if (! py_hostname)
             goto error;
         py_tuple = Py_BuildValue(
-            "(OOOfOi)",
+            "(OOOdOi)",
             py_username,              // username
             py_tty,                   // tty
             py_hostname,              // hostname
-            (float)ut->ut_tv.tv_sec,  // tstamp
+            (double)ut->ut_tv.tv_sec,  // tstamp
             py_user_proc,             // (bool) user process
             ut->ut_pid                // process id
         );
@@ -680,7 +679,7 @@ psutil_net_if_stats(PyObject* self, PyObject* args) {
     if (sock == -1)
         goto error;
 
-    strncpy(ifr.ifr_name, nic_name, sizeof(ifr.ifr_name));
+    PSUTIL_STRNCPY(ifr.ifr_name, nic_name, sizeof(ifr.ifr_name));
 
     // is up?
     ret = ioctl(sock, SIOCGIFFLAGS, &ifr);
@@ -877,7 +876,7 @@ error:
 static PyObject *
 psutil_virtual_mem(PyObject *self, PyObject *args) {
     int rc;
-    int pagesize = getpagesize();
+    long pagesize = psutil_getpagesize();
     perfstat_memory_total_t memory;
 
     rc = perfstat_memory_total(
@@ -903,7 +902,7 @@ psutil_virtual_mem(PyObject *self, PyObject *args) {
 static PyObject *
 psutil_swap_mem(PyObject *self, PyObject *args) {
     int rc;
-    int pagesize = getpagesize();
+    long pagesize = psutil_getpagesize();
     perfstat_memory_total_t memory;
 
     rc = perfstat_memory_total(
@@ -990,58 +989,37 @@ static PyMethodDef
 PsutilMethods[] =
 {
     // --- process-related functions
-    {"proc_basic_info", psutil_proc_basic_info, METH_VARARGS,
-     "Return process ppid, rss, vms, ctime, nice, nthreads, status and tty"},
-    {"proc_name", psutil_proc_name, METH_VARARGS,
-     "Return process name."},
-    {"proc_args", psutil_proc_args, METH_VARARGS,
-     "Return process command line arguments."},
-    {"proc_environ", psutil_proc_environ, METH_VARARGS,
-     "Return process environment variables."},
-    {"proc_cpu_times", psutil_proc_cpu_times, METH_VARARGS,
-     "Return process user and system CPU times."},
-    {"proc_cred", psutil_proc_cred, METH_VARARGS,
-     "Return process uids/gids."},
+    {"proc_args", psutil_proc_args, METH_VARARGS},
+    {"proc_basic_info", psutil_proc_basic_info, METH_VARARGS},
+    {"proc_cpu_times", psutil_proc_cpu_times, METH_VARARGS},
+    {"proc_cred", psutil_proc_cred, METH_VARARGS},
+    {"proc_environ", psutil_proc_environ, METH_VARARGS},
+    {"proc_name", psutil_proc_name, METH_VARARGS},
 #ifdef CURR_VERSION_THREAD
-    {"proc_threads", psutil_proc_threads, METH_VARARGS,
-     "Return process threads"},
+    {"proc_threads", psutil_proc_threads, METH_VARARGS},
 #endif
 #ifdef CURR_VERSION_PROCESS
-    {"proc_io_counters", psutil_proc_io_counters, METH_VARARGS,
-     "Get process I/O counters."},
+    {"proc_io_counters", psutil_proc_io_counters, METH_VARARGS},
 #endif
-    {"proc_num_ctx_switches", psutil_proc_num_ctx_switches, METH_VARARGS,
-     "Get process I/O counters."},
+    {"proc_num_ctx_switches", psutil_proc_num_ctx_switches, METH_VARARGS},
 
     // --- system-related functions
-    {"users", psutil_users, METH_VARARGS,
-     "Return currently connected users."},
-    {"disk_partitions", psutil_disk_partitions, METH_VARARGS,
-     "Return disk partitions."},
-    {"boot_time", psutil_boot_time, METH_VARARGS,
-     "Return system boot time in seconds since the EPOCH."},
-    {"per_cpu_times", psutil_per_cpu_times, METH_VARARGS,
-     "Return system per-cpu times as a list of tuples"},
-    {"disk_io_counters", psutil_disk_io_counters, METH_VARARGS,
-     "Return a Python dict of tuples for disk I/O statistics."},
-    {"virtual_mem", psutil_virtual_mem, METH_VARARGS,
-     "Return system virtual memory usage statistics"},
-    {"swap_mem", psutil_swap_mem, METH_VARARGS,
-     "Return stats about swap memory, in bytes"},
+    {"boot_time", psutil_boot_time, METH_VARARGS},
+    {"disk_io_counters", psutil_disk_io_counters, METH_VARARGS},
+    {"disk_partitions", psutil_disk_partitions, METH_VARARGS},
+    {"per_cpu_times", psutil_per_cpu_times, METH_VARARGS},
+    {"swap_mem", psutil_swap_mem, METH_VARARGS},
+    {"users", psutil_users, METH_VARARGS},
+    {"virtual_mem", psutil_virtual_mem, METH_VARARGS},
 #if defined(CURR_VERSION_NETINTERFACE) && CURR_VERSION_NETINTERFACE >= 3
-    {"net_io_counters", psutil_net_io_counters, METH_VARARGS,
-     "Return a Python dict of tuples for network I/O statistics."},
+    {"net_io_counters", psutil_net_io_counters, METH_VARARGS},
 #endif
-    {"net_connections", psutil_net_connections, METH_VARARGS,
-     "Return system-wide connections"},
-    {"net_if_stats", psutil_net_if_stats, METH_VARARGS,
-     "Return NIC stats (isup, mtu)"},
-    {"cpu_stats", psutil_cpu_stats, METH_VARARGS,
-     "Return CPU statistics"},
+    {"cpu_stats", psutil_cpu_stats, METH_VARARGS},
+    {"net_connections", psutil_net_connections, METH_VARARGS},
+    {"net_if_stats", psutil_net_if_stats, METH_VARARGS},
 
     // --- others
-    {"set_testing", psutil_set_testing, METH_NOARGS,
-     "Set psutil in testing mode"},
+    {"set_debug", psutil_set_debug, METH_VARARGS},
 
     {NULL, NULL, 0, NULL}
 };
