@@ -31,7 +31,7 @@
 #include <sys/param.h>
 #include <sys/sysctl.h>
 #if !defined(__NetBSD__)
-#include <sys/user.h>
+    #include <sys/user.h>
 #endif
 #include <sys/proc.h>
 #include <sys/file.h>
@@ -51,21 +51,23 @@
 #include <netinet/tcp_var.h>   // for struct xtcpcb
 #include <netinet/tcp_fsm.h>   // for TCP connection states
 #include <arpa/inet.h>         // for inet_ntop()
-
 #include <sys/mount.h>
-
 #include <net/if.h>       // net io counters
 #include <net/if_dl.h>
 #include <net/route.h>
-
 #include <netinet/in.h>   // process open files/connections
 #include <sys/un.h>
+#include <kvm.h>
 
 #include "_psutil_common.h"
 #include "_psutil_posix.h"
 
 #ifdef PSUTIL_FREEBSD
-    #include "arch/freebsd/specific.h"
+    #include "arch/freebsd/cpu.h"
+    #include "arch/freebsd/mem.h"
+    #include "arch/freebsd/disk.h"
+    #include "arch/freebsd/sensors.h"
+    #include "arch/freebsd/proc.h"
     #include "arch/freebsd/sys_socks.h"
     #include "arch/freebsd/proc_socks.h"
 
@@ -78,7 +80,11 @@
         #include <utmpx.h>
     #endif
 #elif PSUTIL_OPENBSD
-    #include "arch/openbsd/specific.h"
+    #include "arch/openbsd/cpu.h"
+    #include "arch/openbsd/disk.h"
+    #include "arch/openbsd/mem.h"
+    #include "arch/openbsd/proc.h"
+    #include "arch/openbsd/socks.h"
 
     #include <utmp.h>
     #include <sys/vnode.h>  // for VREG
@@ -87,7 +93,10 @@
     #undef _KERNEL
     #include <sys/sched.h>  // for CPUSTATES & CP_*
 #elif PSUTIL_NETBSD
-    #include "arch/netbsd/specific.h"
+    #include "arch/netbsd/cpu.h"
+    #include "arch/netbsd/disk.h"
+    #include "arch/netbsd/mem.h"
+    #include "arch/netbsd/proc.h"
     #include "arch/netbsd/socks.h"
 
     #include <utmpx.h>
@@ -99,16 +108,9 @@
 #endif
 
 
-
 // convert a timeval struct to a double
 #define PSUTIL_TV2DOUBLE(t) ((t).tv_sec + (t).tv_usec / 1000000.0)
 
-#ifdef PSUTIL_FREEBSD
-    // convert a bintime struct to milliseconds
-    #define PSUTIL_BT2MSEC(bt) (bt.sec * 1000 + (((uint64_t) 1000000000 * \
-                           (uint32_t) (bt.frac >> 32) ) >> 32 ) / 1000000)
-#endif
-
 #if defined(PSUTIL_OPENBSD) || defined (PSUTIL_NETBSD)
     #define PSUTIL_KPT2DOUBLE(t) (t ## _sec + t ## _usec / 1000000.0)
 #endif
@@ -129,26 +131,16 @@ psutil_pids(PyObject *self, PyObject *args) {
     if (py_retlist == NULL)
         return NULL;
 
-    // TODO: RuntimeError is inappropriate here; we could return the
-    // original error instead.
-    if (psutil_get_proc_list(&proclist, &num_processes) != 0) {
-        if (errno != 0) {
-            PyErr_SetFromErrno(PyExc_OSError);
-        }
-        else {
-            PyErr_SetString(PyExc_RuntimeError,
-                            "failed to retrieve process list");
-        }
+    if (psutil_get_proc_list(&proclist, &num_processes) != 0)
         goto error;
-    }
 
     if (num_processes > 0) {
         orig_address = proclist; // save so we can free it after we're done
         for (idx = 0; idx < num_processes; idx++) {
 #ifdef PSUTIL_FREEBSD
-            py_pid = Py_BuildValue("i", proclist->ki_pid);
+            py_pid = PyLong_FromPid(proclist->ki_pid);
 #elif defined(PSUTIL_OPENBSD) || defined(PSUTIL_NETBSD)
-            py_pid = Py_BuildValue("i", proclist->p_pid);
+            py_pid = PyLong_FromPid(proclist->p_pid);
 #endif
             if (!py_pid)
                 goto error;
@@ -194,7 +186,7 @@ psutil_boot_time(PyObject *self, PyObject *args) {
  */
 static PyObject *
 psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
-    long pid;
+    pid_t pid;
     long rss;
     long vms;
     long memtext;
@@ -202,12 +194,13 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
     long memstack;
     int oncpu;
     kinfo_proc kp;
-    long pagesize = sysconf(_SC_PAGESIZE);
+    long pagesize = psutil_getpagesize();
     char str[1000];
     PyObject *py_name;
+    PyObject *py_ppid;
     PyObject *py_retlist;
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, _Py_PARSE_PID, &pid))
         return NULL;
     if (psutil_kinfo_proc(pid, &kp) == -1)
         return NULL;
@@ -269,12 +262,25 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
     oncpu = -1;
 #endif
 
+#ifdef PSUTIL_FREEBSD
+    py_ppid = PyLong_FromPid(kp.ki_ppid);
+#elif defined(PSUTIL_OPENBSD) || defined(PSUTIL_NETBSD)
+    py_ppid = PyLong_FromPid(kp.p_ppid);
+#else
+    py_ppid = Py_BuildfValue(-1);
+#endif
+    if (! py_ppid)
+        return NULL;
+
     // Return a single big tuple with all process info.
     py_retlist = Py_BuildValue(
-        "(lillllllidllllddddlllllbO)",
+#if defined(__FreeBSD_version) && __FreeBSD_version >= 1200031
+        "(OillllllLdllllddddlllllbO)",
+#else
+        "(OillllllidllllddddlllllbO)",
+#endif
 #ifdef PSUTIL_FREEBSD
-        //
-        (long)kp.ki_ppid,                // (long) ppid
+        py_ppid,                         // (pid_t) ppid
         (int)kp.ki_stat,                 // (int) status
         // UIDs
         (long)kp.ki_ruid,                // (long) real uid
@@ -285,7 +291,7 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
         (long)kp.ki_groups[0],           // (long) effective gid
         (long)kp.ki_svuid,               // (long) saved gid
         //
-        kp.ki_tdev,                      // (int) tty nr
+        kp.ki_tdev,                      // (int or long long) tty nr
         PSUTIL_TV2DOUBLE(kp.ki_start),   // (double) create time
         // ctx switches
         kp.ki_rusage.ru_nvcsw,           // (long) ctx switches (voluntary)
@@ -307,8 +313,7 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
         // others
         oncpu,                            // (int) the CPU we are on
 #elif defined(PSUTIL_OPENBSD) || defined(PSUTIL_NETBSD)
-        //
-        (long)kp.p_ppid,                 // (long) ppid
+        py_ppid,                         // (pid_t) ppid
         (int)kp.p_stat,                  // (int) status
         // UIDs
         (long)kp.p_ruid,                 // (long) real uid
@@ -347,6 +352,7 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
     );
 
     Py_DECREF(py_name);
+    Py_DECREF(py_ppid);
     return py_retlist;
 }
 
@@ -356,11 +362,11 @@ psutil_proc_oneshot_info(PyObject *self, PyObject *args) {
  */
 static PyObject *
 psutil_proc_name(PyObject *self, PyObject *args) {
-    long pid;
+    pid_t pid;
     kinfo_proc kp;
     char str[1000];
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, _Py_PARSE_PID, &pid))
         return NULL;
     if (psutil_kinfo_proc(pid, &kp) == -1)
         return NULL;
@@ -379,10 +385,10 @@ psutil_proc_name(PyObject *self, PyObject *args) {
  */
 static PyObject *
 psutil_proc_cmdline(PyObject *self, PyObject *args) {
-    long pid;
+    pid_t pid;
     PyObject *py_retlist = NULL;
 
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, _Py_PARSE_PID, &pid))
         return NULL;
     py_retlist = psutil_get_cmdline(pid);
     if (py_retlist == NULL)
@@ -391,6 +397,140 @@ psutil_proc_cmdline(PyObject *self, PyObject *args) {
 }
 
 
+/*
+ * Return process environment as a Python dictionary
+ */
+PyObject *
+psutil_proc_environ(PyObject *self, PyObject *args) {
+    int i, cnt = -1;
+    long pid;
+    char *s, **envs, errbuf[_POSIX2_LINE_MAX];
+    PyObject *py_value=NULL, *py_retdict=NULL;
+    kvm_t *kd;
+#ifdef PSUTIL_NETBSD
+    struct kinfo_proc2 *p;
+#else
+    struct kinfo_proc *p;
+#endif
+
+    if (!PyArg_ParseTuple(args, "l", &pid))
+        return NULL;
+
+#if defined(PSUTIL_FREEBSD)
+    kd = kvm_openfiles(NULL, "/dev/null", NULL, 0, errbuf);
+#else
+    kd = kvm_openfiles(NULL, NULL, NULL, KVM_NO_FILES, errbuf);
+#endif
+    if (!kd) {
+        convert_kvm_err("kvm_openfiles", errbuf);
+        return NULL;
+    }
+
+    py_retdict = PyDict_New();
+    if (!py_retdict)
+        goto error;
+
+#if defined(PSUTIL_FREEBSD)
+    p = kvm_getprocs(kd, KERN_PROC_PID, pid, &cnt);
+#elif defined(PSUTIL_OPENBSD)
+    p = kvm_getprocs(kd, KERN_PROC_PID, pid, sizeof(*p), &cnt);
+#elif defined(PSUTIL_NETBSD)
+    p = kvm_getproc2(kd, KERN_PROC_PID, pid, sizeof(*p), &cnt);
+#endif
+    if (!p) {
+        NoSuchProcess("kvm_getprocs");
+        goto error;
+    }
+    if (cnt <= 0) {
+        NoSuchProcess(cnt < 0 ? kvm_geterr(kd) : "kvm_getprocs: cnt==0");
+        goto error;
+    }
+
+    // On *BSD kernels there are a few kernel-only system processes without an
+    // environment (See e.g. "procstat -e 0 | 1 | 2 ..." on FreeBSD.)
+    // Some system process have no stats attached at all
+    // (they are marked with P_SYSTEM.)
+    // On FreeBSD, it's possible that the process is swapped or paged out,
+    // then there no access to the environ stored in the process' user area.
+    // On NetBSD, we cannot call kvm_getenvv2() for a zombie process.
+    // To make unittest suite happy, return an empty environment.
+#if defined(PSUTIL_FREEBSD)
+#if (defined(__FreeBSD_version) && __FreeBSD_version >= 700000)
+    if (!((p)->ki_flag & P_INMEM) || ((p)->ki_flag & P_SYSTEM)) {
+#else
+    if ((p)->ki_flag & P_SYSTEM) {
+#endif
+#elif defined(PSUTIL_NETBSD)
+    if ((p)->p_stat == SZOMB) {
+#elif defined(PSUTIL_OPENBSD)
+    if ((p)->p_flag & P_SYSTEM) {
+#endif
+        kvm_close(kd);
+        return py_retdict;
+    }
+
+#if defined(PSUTIL_NETBSD)
+    envs = kvm_getenvv2(kd, p, 0);
+#else
+    envs = kvm_getenvv(kd, p, 0);
+#endif
+    if (!envs) {
+        // Map to "psutil" general high-level exceptions
+        switch (errno) {
+            case 0:
+                // Process has cleared it's environment, return empty one
+                kvm_close(kd);
+                return py_retdict;
+            case EPERM:
+                AccessDenied("kvm_getenvv -> EPERM");
+                break;
+            case ESRCH:
+                NoSuchProcess("kvm_getenvv -> ESRCH");
+                break;
+#if defined(PSUTIL_FREEBSD)
+            case ENOMEM:
+                // Unfortunately, under FreeBSD kvm_getenvv() returns
+                // failure for certain processes ( e.g. try
+                // "sudo procstat -e <pid of your XOrg server>".)
+                // Map the error condition to 'AccessDenied'.
+                sprintf(errbuf,
+                        "kvm_getenvv(pid=%ld, ki_uid=%d) -> ENOMEM",
+                        pid, p->ki_uid);
+                AccessDenied(errbuf);
+                break;
+#endif
+            default:
+                sprintf(errbuf, "kvm_getenvv(pid=%ld)", pid);
+                PyErr_SetFromOSErrnoWithSyscall(errbuf);
+                break;
+        }
+        goto error;
+    }
+
+    for (i = 0; envs[i] != NULL; i++) {
+        s = strchr(envs[i], '=');
+        if (!s)
+            continue;
+        *s++ = 0;
+        py_value = PyUnicode_DecodeFSDefault(s);
+        if (!py_value)
+            goto error;
+        if (PyDict_SetItemString(py_retdict, envs[i], py_value)) {
+            goto error;
+        }
+        Py_DECREF(py_value);
+    }
+
+    kvm_close(kd);
+    return py_retdict;
+
+error:
+    Py_XDECREF(py_value);
+    Py_XDECREF(py_retdict);
+    kvm_close(kd);
+    return NULL;
+}
+
 /*
  * Return the number of logical CPUs in the system.
  * XXX this could be shared with macOS
@@ -452,7 +592,7 @@ psutil_cpu_times(PyObject *self, PyObject *args) {
 #if (defined(__FreeBSD_version) && __FreeBSD_version >= 800000) || PSUTIL_OPENBSD || defined(PSUTIL_NETBSD)
 static PyObject *
 psutil_proc_open_files(PyObject *self, PyObject *args) {
-    long pid;
+    pid_t pid;
     int i;
     int cnt;
     int regular;
@@ -467,7 +607,7 @@ psutil_proc_open_files(PyObject *self, PyObject *args) {
 
     if (py_retlist == NULL)
         return NULL;
-    if (! PyArg_ParseTuple(args, "l", &pid))
+    if (! PyArg_ParseTuple(args, _Py_PARSE_PID, &pid))
         goto error;
     if (psutil_kinfo_proc(pid, &kipp) == -1)
         goto error;
@@ -475,7 +615,9 @@ psutil_proc_open_files(PyObject *self, PyObject *args) {
     errno = 0;
     freep = kinfo_getfile(pid, &cnt);
     if (freep == NULL) {
+#if !defined(PSUTIL_OPENBSD)
         psutil_raise_for_pid(pid, "kinfo_getfile()");
+#endif
         goto error;
     }
 
@@ -615,8 +757,10 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
             strlcat(opts, ",softdep", sizeof(opts));
         if (flags & MNT_NOSYMFOLLOW)
             strlcat(opts, ",nosymfollow", sizeof(opts));
+#ifdef MNT_GJOURNAL
         if (flags & MNT_GJOURNAL)
             strlcat(opts, ",gjournal", sizeof(opts));
+#endif
         if (flags & MNT_MULTILABEL)
             strlcat(opts, ",multilabel", sizeof(opts));
         if (flags & MNT_ACLS)
@@ -625,8 +769,10 @@ psutil_disk_partitions(PyObject *self, PyObject *args) {
             strlcat(opts, ",noclusterr", sizeof(opts));
         if (flags & MNT_NOCLUSTERW)
             strlcat(opts, ",noclusterw", sizeof(opts));
+#ifdef MNT_NFS4ACLS
         if (flags & MNT_NFS4ACLS)
             strlcat(opts, ",nfs4acls", sizeof(opts));
+#endif
 #elif PSUTIL_NETBSD
         if (flags & MNT_NODEV)
             strlcat(opts, ",nodev", sizeof(opts));
@@ -794,6 +940,7 @@ psutil_users(PyObject *self, PyObject *args) {
     PyObject *py_tty = NULL;
     PyObject *py_hostname = NULL;
     PyObject *py_tuple = NULL;
+    PyObject *py_pid = NULL;
 
     if (py_retlist == NULL)
         return NULL;
@@ -823,15 +970,15 @@ psutil_users(PyObject *self, PyObject *args) {
         if (! py_hostname)
             goto error;
         py_tuple = Py_BuildValue(
-            "(OOOfi)",
+            "(OOOdi)",
             py_username,        // username
             py_tty,             // tty
             py_hostname,        // hostname
-            (float)ut.ut_time,  // start time
-#ifdef PSUTIL_OPENBSD
+            (double)ut.ut_time,  // start time
+#if defined(PSUTIL_OPENBSD) || (defined(__FreeBSD_version) && __FreeBSD_version < 900000)
             -1                  // process id (set to None later)
 #else
-            ut.ut_pid           // process id
+            ut.ut_pid           // TODO: use PyLong_FromPid
 #endif
         );
         if (!py_tuple) {
@@ -864,17 +1011,21 @@ psutil_users(PyObject *self, PyObject *args) {
         py_hostname = PyUnicode_DecodeFSDefault(utx->ut_host);
         if (! py_hostname)
             goto error;
+#ifdef PSUTIL_OPENBSD
+        py_pid = Py_BuildValue("i", -1);  // set to None later
+#else
+        py_pid = PyLong_FromPid(utx->ut_pid);
+#endif
+        if (! py_pid)
+            goto error;
+
         py_tuple = Py_BuildValue(
-            "(OOOfi)",
+            "(OOOdO)",
             py_username,   // username
             py_tty,        // tty
             py_hostname,   // hostname
-            (float)utx->ut_tv.tv_sec,  // start time
-#ifdef PSUTIL_OPENBSD
-            -1             // process id (set to None later)
-#else
-            utx->ut_pid    // process id
-#endif
+            (double)utx->ut_tv.tv_sec,  // start time
+            py_pid         // process id
         );
 
         if (!py_tuple) {
@@ -889,6 +1040,7 @@ psutil_users(PyObject *self, PyObject *args) {
         Py_CLEAR(py_tty);
         Py_CLEAR(py_hostname);
         Py_CLEAR(py_tuple);
+        Py_CLEAR(py_pid);
     }
 
     endutxent();
@@ -900,6 +1052,7 @@ error:
     Py_XDECREF(py_tty);
     Py_XDECREF(py_hostname);
     Py_XDECREF(py_tuple);
+    Py_XDECREF(py_pid);
     Py_DECREF(py_retlist);
     return NULL;
 }
@@ -911,86 +1064,55 @@ error:
 static PyMethodDef mod_methods[] = {
     // --- per-process functions
 
-    {"proc_oneshot_info", psutil_proc_oneshot_info, METH_VARARGS,
-     "Return multiple info about a process"},
-    {"proc_name", psutil_proc_name, METH_VARARGS,
-     "Return process name"},
-    {"proc_cmdline", psutil_proc_cmdline, METH_VARARGS,
-     "Return process cmdline as a list of cmdline arguments"},
-    {"proc_threads", psutil_proc_threads, METH_VARARGS,
-     "Return process threads"},
-#if defined(PSUTIL_FREEBSD) || defined(PSUTIL_OPENBSD)
-    {"proc_connections", psutil_proc_connections, METH_VARARGS,
-     "Return connections opened by process"},
+    {"proc_cmdline", psutil_proc_cmdline, METH_VARARGS},
+    {"proc_name", psutil_proc_name, METH_VARARGS},
+    {"proc_oneshot_info", psutil_proc_oneshot_info, METH_VARARGS},
+    {"proc_threads", psutil_proc_threads, METH_VARARGS},
+#if defined(PSUTIL_FREEBSD)
+    {"proc_connections", psutil_proc_connections, METH_VARARGS},
 #endif
-    {"proc_cwd", psutil_proc_cwd, METH_VARARGS,
-     "Return process current working directory."},
+    {"proc_cwd", psutil_proc_cwd, METH_VARARGS},
 #if defined(__FreeBSD_version) && __FreeBSD_version >= 800000 || PSUTIL_OPENBSD || defined(PSUTIL_NETBSD)
-    {"proc_num_fds", psutil_proc_num_fds, METH_VARARGS,
-     "Return the number of file descriptors opened by this process"},
-    {"proc_open_files", psutil_proc_open_files, METH_VARARGS,
-     "Return files opened by process as a list of (path, fd) tuples"},
+    {"proc_num_fds", psutil_proc_num_fds, METH_VARARGS},
+    {"proc_open_files", psutil_proc_open_files, METH_VARARGS},
 #endif
 #if defined(PSUTIL_FREEBSD) || defined(PSUTIL_NETBSD)
-    {"proc_num_threads", psutil_proc_num_threads, METH_VARARGS,
-     "Return number of threads used by process"},
+    {"proc_num_threads", psutil_proc_num_threads, METH_VARARGS},
 #endif
 #if defined(PSUTIL_FREEBSD)
-    {"proc_exe", psutil_proc_exe, METH_VARARGS,
-     "Return process pathname executable"},
-    {"proc_memory_maps", psutil_proc_memory_maps, METH_VARARGS,
-     "Return a list of tuples for every process's memory map"},
-    {"proc_cpu_affinity_get", psutil_proc_cpu_affinity_get, METH_VARARGS,
-     "Return process CPU affinity."},
-    {"proc_cpu_affinity_set", psutil_proc_cpu_affinity_set, METH_VARARGS,
-     "Set process CPU affinity."},
-    {"cpu_count_phys", psutil_cpu_count_phys, METH_VARARGS,
-     "Return an XML string to determine the number physical CPUs."},
+    {"cpu_topology", psutil_cpu_topology, METH_VARARGS},
+    {"proc_cpu_affinity_get", psutil_proc_cpu_affinity_get, METH_VARARGS},
+    {"proc_cpu_affinity_set", psutil_proc_cpu_affinity_set, METH_VARARGS},
+    {"proc_exe", psutil_proc_exe, METH_VARARGS},
+    {"proc_getrlimit", psutil_proc_getrlimit, METH_VARARGS},
+    {"proc_memory_maps", psutil_proc_memory_maps, METH_VARARGS},
+    {"proc_setrlimit", psutil_proc_setrlimit, METH_VARARGS},
 #endif
+    {"proc_environ", psutil_proc_environ, METH_VARARGS},
 
     // --- system-related functions
-
-    {"pids", psutil_pids, METH_VARARGS,
-     "Returns a list of PIDs currently running on the system"},
-    {"cpu_count_logical", psutil_cpu_count_logical, METH_VARARGS,
-     "Return number of logical CPUs on the system"},
-    {"virtual_mem", psutil_virtual_mem, METH_VARARGS,
-     "Return system virtual memory usage statistics"},
-    {"swap_mem", psutil_swap_mem, METH_VARARGS,
-     "Return swap mem stats"},
-    {"cpu_times", psutil_cpu_times, METH_VARARGS,
-     "Return system cpu times as a tuple (user, system, nice, idle, irc)"},
-    {"per_cpu_times", psutil_per_cpu_times, METH_VARARGS,
-     "Return system per-cpu times as a list of tuples"},
-    {"boot_time", psutil_boot_time, METH_VARARGS,
-     "Return the system boot time expressed in seconds since the epoch."},
-    {"disk_partitions", psutil_disk_partitions, METH_VARARGS,
-     "Return a list of tuples including device, mount point and "
-     "fs type for all partitions mounted on the system."},
-    {"net_io_counters", psutil_net_io_counters, METH_VARARGS,
-     "Return dict of tuples of networks I/O information."},
-    {"disk_io_counters", psutil_disk_io_counters, METH_VARARGS,
-     "Return a Python dict of tuples for disk I/O information"},
-    {"users", psutil_users, METH_VARARGS,
-     "Return currently connected users as a list of tuples"},
-    {"cpu_stats", psutil_cpu_stats, METH_VARARGS,
-     "Return CPU statistics"},
-#if defined(PSUTIL_FREEBSD) || defined(PSUTIL_NETBSD)
-    {"net_connections", psutil_net_connections, METH_VARARGS,
-     "Return system-wide open connections."},
+    {"boot_time", psutil_boot_time, METH_VARARGS},
+    {"cpu_count_logical", psutil_cpu_count_logical, METH_VARARGS},
+    {"cpu_stats", psutil_cpu_stats, METH_VARARGS},
+    {"cpu_times", psutil_cpu_times, METH_VARARGS},
+    {"disk_io_counters", psutil_disk_io_counters, METH_VARARGS},
+    {"disk_partitions", psutil_disk_partitions, METH_VARARGS},
+    {"net_connections", psutil_net_connections, METH_VARARGS},
+    {"net_io_counters", psutil_net_io_counters, METH_VARARGS},
+    {"per_cpu_times", psutil_per_cpu_times, METH_VARARGS},
+    {"pids", psutil_pids, METH_VARARGS},
+    {"swap_mem", psutil_swap_mem, METH_VARARGS},
+    {"users", psutil_users, METH_VARARGS},
+    {"virtual_mem", psutil_virtual_mem, METH_VARARGS},
+#if defined(PSUTIL_FREEBSD) || defined(PSUTIL_OPENBSD)
+     {"cpu_freq", psutil_cpu_freq, METH_VARARGS},
 #endif
 #if defined(PSUTIL_FREEBSD)
-    {"sensors_battery", psutil_sensors_battery, METH_VARARGS,
-     "Return battery information."},
-    {"sensors_cpu_temperature", psutil_sensors_cpu_temperature, METH_VARARGS,
-     "Return temperature information for a given CPU core number."},
-    {"cpu_frequency", psutil_cpu_freq, METH_VARARGS,
-     "Return frequency of a given CPU"},
+    {"sensors_battery", psutil_sensors_battery, METH_VARARGS},
+    {"sensors_cpu_temperature", psutil_sensors_cpu_temperature, METH_VARARGS},
 #endif
-
     // --- others
-    {"set_testing", psutil_set_testing, METH_NOARGS,
-     "Set psutil in testing mode"},
+    {"set_debug", psutil_set_debug, METH_VARARGS},
 
     {NULL, NULL, 0, NULL}
 };
@@ -1051,7 +1173,9 @@ static PyMethodDef mod_methods[] = {
     if (PyModule_AddIntConstant(mod, "SSLEEP", LSSLEEP)) INITERR;
     if (PyModule_AddIntConstant(mod, "SSTOP", LSSTOP)) INITERR;
     if (PyModule_AddIntConstant(mod, "SZOMB", LSZOMB)) INITERR;
+#if __NetBSD_Version__ < 500000000
     if (PyModule_AddIntConstant(mod, "SDEAD", LSDEAD)) INITERR;
+#endif
     if (PyModule_AddIntConstant(mod, "SONPROC", LSONPROC)) INITERR;
     // unique to NetBSD
     if (PyModule_AddIntConstant(mod, "SSUSPENDED", LSSUSPENDED)) INITERR;
