@@ -12,6 +12,7 @@ Some of these are duplicates of tests test_system.py and test_process.py
 import errno
 import multiprocessing
 import os
+import platform
 import signal
 import stat
 import sys
@@ -34,6 +35,7 @@ from psutil import WINDOWS
 from psutil._compat import FileNotFoundError
 from psutil._compat import long
 from psutil._compat import range
+from psutil._compat import unicode
 from psutil.tests import APPVEYOR
 from psutil.tests import CI_TESTING
 from psutil.tests import GITHUB_ACTIONS
@@ -174,7 +176,7 @@ class TestAvailProcessAPIs(PsutilTestCase):
 
     def test_io_counters(self):
         hasit = hasattr(psutil.Process, "io_counters")
-        self.assertEqual(hasit, False if MACOS or SUNOS else True)
+        self.assertEqual(hasit, not (MACOS or SUNOS))
 
     def test_num_fds(self):
         self.assertEqual(hasattr(psutil.Process, "num_fds"), POSIX)
@@ -234,6 +236,9 @@ class TestSystemAPITypes(PsutilTestCase):
     def test_cpu_count(self):
         self.assertIsInstance(psutil.cpu_count(), int)
 
+    # TODO: remove this once 1892 is fixed
+    @unittest.skipIf(MACOS and platform.machine() == 'arm64',
+                     "skipped due to #1892")
     @unittest.skipIf(not HAS_CPU_FREQ, "not supported")
     def test_cpu_freq(self):
         if psutil.cpu_freq() is None:
@@ -253,8 +258,8 @@ class TestSystemAPITypes(PsutilTestCase):
             self.assertIsInstance(disk.mountpoint, str)
             self.assertIsInstance(disk.fstype, str)
             self.assertIsInstance(disk.opts, str)
-            self.assertIsInstance(disk.maxfile, int)
-            self.assertIsInstance(disk.maxpath, int)
+            self.assertIsInstance(disk.maxfile, (int, type(None)))
+            self.assertIsInstance(disk.maxpath, (int, type(None)))
 
     @unittest.skipIf(SKIP_SYSCONS, "requires root")
     def test_net_connections(self):
@@ -397,17 +402,28 @@ class TestFetchAllProcesses(PsutilTestCase):
     """
 
     def setUp(self):
-        self.pool = multiprocessing.Pool()
+        # Using a pool in a CI env may result in deadlock, see:
+        # https://github.com/giampaolo/psutil/issues/2104
+        if not CI_TESTING:
+            self.pool = multiprocessing.Pool()
 
     def tearDown(self):
-        self.pool.terminate()
-        self.pool.join()
+        if not CI_TESTING:
+            self.pool.terminate()
+            self.pool.join()
 
     def iter_proc_info(self):
         # Fixes "can't pickle <function proc_info>: it's not the
         # same object as test_contracts.proc_info".
         from psutil.tests.test_contracts import proc_info
-        return self.pool.imap_unordered(proc_info, psutil.pids())
+
+        if not CI_TESTING:
+            return self.pool.imap_unordered(proc_info, psutil.pids())
+        else:
+            ls = []
+            for pid in psutil.pids():
+                ls.append(proc_info(pid))
+            return ls
 
     def test_all(self):
         failures = []
@@ -422,8 +438,7 @@ class TestFetchAllProcesses(PsutilTestCase):
                         name, info['pid'], repr(value))
                     s += '-' * 70
                     s += "\n%s" % traceback.format_exc()
-                    s = "\n".join((" " * 4) + i for i in s.splitlines())
-                    s += '\n'
+                    s = "\n".join((" " * 4) + i for i in s.splitlines()) + "\n"
                     failures.append(s)
                 else:
                     if value not in (0, 0.0, [], None, '', {}):
@@ -437,10 +452,9 @@ class TestFetchAllProcesses(PsutilTestCase):
             self.assertIsInstance(part, str)
 
     def exe(self, ret, info):
-        self.assertIsInstance(ret, (str, type(None)))
-        if not ret:
-            self.assertEqual(ret, '')
-        else:
+        self.assertIsInstance(ret, (str, unicode))
+        self.assertEqual(ret.strip(), ret)
+        if ret:
             if WINDOWS and not ret.endswith('.exe'):
                 return  # May be "Registry", "MemCompression", ...
             assert os.path.isabs(ret), ret
@@ -465,7 +479,7 @@ class TestFetchAllProcesses(PsutilTestCase):
         self.assertGreaterEqual(ret, 0)
 
     def name(self, ret, info):
-        self.assertIsInstance(ret, str)
+        self.assertIsInstance(ret, (str, unicode))
         if APPVEYOR and not ret and info['status'] == 'stopped':
             return
         # on AIX, "<exiting>" processes don't have names
@@ -505,7 +519,8 @@ class TestFetchAllProcesses(PsutilTestCase):
 
     def username(self, ret, info):
         self.assertIsInstance(ret, str)
-        assert ret
+        self.assertEqual(ret.strip(), ret)
+        assert ret.strip()
 
     def status(self, ret, info):
         self.assertIsInstance(ret, str)
@@ -604,6 +619,7 @@ class TestFetchAllProcesses(PsutilTestCase):
         for f in ret:
             self.assertIsInstance(f.fd, int)
             self.assertIsInstance(f.path, str)
+            self.assertEqual(f.path.strip(), f.path)
             if WINDOWS:
                 self.assertEqual(f.fd, -1)
             elif LINUX:
@@ -636,8 +652,9 @@ class TestFetchAllProcesses(PsutilTestCase):
                 check_connection_ntuple(conn)
 
     def cwd(self, ret, info):
-        if ret:     # 'ret' can be None or empty
-            self.assertIsInstance(ret, str)
+        self.assertIsInstance(ret, (str, unicode))
+        self.assertEqual(ret.strip(), ret)
+        if ret:
             assert os.path.isabs(ret), ret
             try:
                 st = os.stat(ret)
@@ -680,7 +697,7 @@ class TestFetchAllProcesses(PsutilTestCase):
             for fname in nt._fields:
                 value = getattr(nt, fname)
                 if fname == 'path':
-                    if not value.startswith('['):
+                    if not value.startswith(("[", "anon_inode:")):
                         assert os.path.isabs(nt.path), nt.path
                         # commented as on Linux we might get
                         # '/foo/bar (deleted)'
