@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
@@ -13,6 +13,7 @@ asus
 """
 
 from __future__ import print_function
+
 import sys
 
 import psutil
@@ -23,7 +24,8 @@ def main():
         return sys.exit("platform not supported")
     fans = psutil.sensors_fans()
     if not fans:
-        return sys.exit("no fans detected")
+        print("no fans detected")
+        return
     for name, entries in fans.items():
         print(name)
         for entry in entries:
