@@ -82,7 +82,7 @@ DEFAULT_COLOR = 7
 # ===================================================================
 
 
-def safe_print(text, file=sys.stdout, flush=False):
+def safe_print(text, file=sys.stdout):
     """Prints a (unicode) string to the console, encoded depending on
     the stdout/file encoding (eg. cp437 on Windows). This is to avoid
     encoding errors in case of funky path names.
@@ -228,7 +228,7 @@ def build():
     # order to allow "import psutil" when using the interactive interpreter
     # from within psutil root directory.
     cmd = [PYTHON, "setup.py", "build_ext", "-i"]
-    if sys.version_info[:2] >= (3, 6) and os.cpu_count() or 1 > 1:
+    if sys.version_info[:2] >= (3, 6) and (os.cpu_count() or 1) > 1:
         cmd += ['--parallel', str(os.cpu_count())]
     # Print coloured warnings in real time.
     p = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
@@ -383,7 +383,7 @@ def setup_dev_env():
     sh("%s -m pip install -U %s" % (PYTHON, " ".join(DEPS)))
 
 
-def check_flake8():
+def flake8():
     """Run flake8 against all py files"""
     py_files = subprocess.check_output("git ls-files")
     if PY3:
@@ -547,8 +547,7 @@ def get_python(path):
             return pypath
 
 
-def main():
-    global PYTHON
+def parse_args():
     parser = argparse.ArgumentParser()
     # option shared by all commands
     parser.add_argument(
@@ -565,7 +564,7 @@ def main():
     sp.add_parser('install', help="build + install in develop/edit mode")
     sp.add_parser('install-git-hooks', help="install GIT pre-commit hook")
     sp.add_parser('install-pip', help="install pip")
-    sp.add_parser('check_flake8', help="run flake8 against all py files")
+    sp.add_parser('flake8', help="run flake8 against all py files")
     sp.add_parser('print-access-denied', help="print AD exceptions")
     sp.add_parser('print-api-speed', help="benchmark all API calls")
     sp.add_parser('setup-dev-env', help="install deps")
@@ -587,8 +586,19 @@ def main():
 
     for p in (test, test_by_name):
         p.add_argument('arg', type=str, nargs='?', default="", help="arg")
+
     args = parser.parse_args()
 
+    if not args.command or args.command == 'help':
+        parser.print_help(sys.stderr)
+        sys.exit(1)
+
+    return args
+
+
+def main():
+    global PYTHON
+    args = parse_args()
     # set python exe
     PYTHON = get_python(args.python)
     if not PYTHON:
@@ -597,10 +607,6 @@ def main():
     os.putenv('PYTHON', PYTHON)
     win_colorprint("using " + PYTHON)
 
-    if not args.command or args.command == 'help':
-        parser.print_help(sys.stderr)
-        sys.exit(1)
-
     fname = args.command.replace('-', '_')
     fun = getattr(sys.modules[__name__], fname)  # err if fun not defined
     funargs = []
