@@ -1,4 +1,4 @@
-#!/usr/bin/env python
+#!/usr/bin/env python3
 
 # Copyright (c) 2009, Giampaolo Rodola', karthikrev. All rights reserved.
 # Use of this source code is governed by a BSD-style license that can be
@@ -12,31 +12,19 @@ $ pidof python
 """
 
 from __future__ import print_function
-import psutil
+
 import sys
 
+import psutil
+
 
 def pidof(pgname):
     pids = []
-    for proc in psutil.process_iter():
+    for proc in psutil.process_iter(['name', 'cmdline']):
         # search for matches in the process name and cmdline
-        try:
-            name = proc.name()
-        except psutil.Error:
-            pass
-        else:
-            if name == pgname:
-                pids.append(str(proc.pid))
-                continue
-
-        try:
-            cmdline = proc.cmdline()
-        except psutil.Error:
-            pass
-        else:
-            if cmdline and cmdline[0] == pgname:
-                pids.append(str(proc.pid))
-
+        if proc.info['name'] == pgname or \
+                proc.info['cmdline'] and proc.info['cmdline'][0] == pgname:
+            pids.append(str(proc.pid))
     return pids
 
 
